//MSQ-CC-LRE-V1=======================================================================//
//======Synth-Programmer with CC-Automation==========================================//
//============2015 by Michael Sigl aka "Phatline,GreatFullTekk,GuteSigl,Technomiliz..."//
//non-commercial use only. All other rights reserved. Mios32 Based > www.ucapps.de=====//
//=====================================================================================//

//NORD-DRUM 2 ---- EDITON//

#include <mios32.h>
#include "app.h"
#include "tasks.h"
#include <FreeRTOS.h>
#include <portmacro.h>
#include <task.h>
#include <queue.h>
#include <semphr.h>
#include "tasks.h"
#include "file.h"
#include <string.h>
#include <stdio.h>
#include <seq_bpm.h>
#include <wchar.h> // to copy pattern-memory

// when you read somewhere "505" then this is a notice that i have changed something, in there is a place too look on debug
// when you read somewhere "303" then something has to be done - future features and so on.

xSemaphoreHandle xSDCardSemaphore; 	// take and give access to SD-Card



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//System Variables
////////////////////////////////////////////////

    // Note, Clock, PC  SOURCE  &&  DESTINATION
    #define  PCChIN 6  //ProgramChange Input Channel
    #define  CC_PC_Bank 32 // Bank for Program-Change in order to use more then 128 songs....
    #define  CC_Tact_mono 23 // 3/4,   4/4 tact system receive for external sequencers
    #define  CC_Save_mono 24 // external store signal, so this app save @ same time like external (only one save button in setup...)
    #define  MeloPort 32 //connect there @IN SEQUENCER    @OUT SYNTHESIZERs ... it also handles ProgramChange
    const    u8 MeloCh[8]   = {6,7,8,9,10,11,4,5};   // receive Triggers (4 Melody Morph)  &  send the CCs to this CHannels  //ACT ALSO AS MIDIFILTER! avoid to much traffic!

    #define  DumpOutRateSynth 1 //starting with 0>1x34ms, 1>2*34=68ms, 2>3*34=102ms 3>4x34=136ms....every 2ms one of 34 CCs on 8 MidiChannels is dumped out
    
    #define  TrigMidiThruClock 1 //MeloPort send Midiclock reroute that to the UART-Outputs
  
    #define  SD_Check_Time 131072       // (us?)
    #define  PC_Load_Timer 250          // (ms)  //505 once was 500
    #define  PC_Save_Timer 250          // (ms)  //505 once was 500

    // Task Prioritys	
    #define PRIORITY_MSQ	            ( tskIDLE_PRIORITY + 2 ) //4:high            
    #define PRIORITY_APP_Tick	        ( tskIDLE_PRIORITY + 2 ) //2 lower then midi
    #define PRIORITY_TASK_SEQ		    ( tskIDLE_PRIORITY + 4 ) // higher priority than MIDI receive task!
    #define PRIORITY_TASK_FLAG		    ( tskIDLE_PRIORITY + 2 ) // higher priority than MIDI receive task!
    
    // local prototype of the task function
    static void TASK_FLAG(void *pvParameters);
    void SD(u16 clip, u16 job);
    static void TASK_SEQ(void *pvParameters);
    static s32 NOTIFY_MIDI_Rx(mios32_midi_port_t port, u8 byte);

    u8 CC_Tact_Ext_Storage = 0;
    
    
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//CC Mapping- Internal > External
/////////////////////////////////
// What is IO Mapping: short: >>> PRESET >>> Routing! --- make it Fit to your SYNTHs! 
// I = Internal >  [32] Array Program Internal
// O = External >  SYNTH

  u8 CC_Map_Mode = 0;   // 0:Deep Edit: ALL Automatable controlls for 1 Channel 
                        //             32 Encoders representing a Single Part.  
                        // 1:Channel Strip: 4 Controls on 8 Channels
                        //             8x4 Vertical Rows, each Row representing 4CCs of one of the 8 Parts... take it as a Mixer-overview... 4 the most importent CCs 

    // 4 CC_Route_Mode = 0:     =  1x Syntheditor             
    const u8 CC_Map0[8][32] = { // CC_Map0 [Part]        [Internal CC Nr] = value of external CC   >>
                                // CC_Map1 [MidiChannel] [Remote/Source]  = Value of Synth/Destination
              //  1st Row Horizontal      (top-row)               //  2nd Row Horizontal                             //  3rd Row Horizontal                              //  4th Row Horizontal
 // CC-on-LRE:  0     1     2     3     4     5     6     7         8     9    10    11    12    13    14    15       16    17    18    19    20    21    22    23        24    25    26    27    28    29    30    31  
            {  57 ,  56 ,  24 ,  23 ,  26 ,  25 ,  10 ,   7 ,      15 ,  17 ,  14 ,  16 ,  18 ,  19 ,  20 ,  21 ,     46 ,  52 ,  53 ,  47 ,  48 ,  54 ,  55 , 254 ,      50 ,  51 ,  49 ,  30 ,  58 , 255 , 255 , 255} ,    // Nord Drum 2    Voice 1   MidiCh  7
            {  57 ,  56 ,  24 ,  23 ,  26 ,  25 ,  10 ,   7 ,      15 ,  17 ,  14 ,  16 ,  18 ,  19 ,  20 ,  21 ,     46 ,  52 ,  53 ,  47 ,  48 ,  54 ,  55 , 254 ,      50 ,  51 ,  49 ,  30 ,  58 , 255 , 255 , 255} ,    // Nord Drum 2    Voice 2   MidiCh  8
            {  57 ,  56 ,  24 ,  23 ,  26 ,  25 ,  10 ,   7 ,      15 ,  17 ,  14 ,  16 ,  18 ,  19 ,  20 ,  21 ,     46 ,  52 ,  53 ,  47 ,  48 ,  54 ,  55 , 254 ,      50 ,  51 ,  49 ,  30 ,  58 , 255 , 255 , 255} ,    // Nord Drum 2    Voice 3   MidiCH  9
            {  57 ,  56 ,  24 ,  23 ,  26 ,  25 ,  10 ,   7 ,      15 ,  17 ,  14 ,  16 ,  18 ,  19 ,  20 ,  21 ,     46 ,  52 ,  53 ,  47 ,  48 ,  54 ,  55 , 254 ,      50 ,  51 ,  49 ,  30 ,  58 , 255 , 255 , 255} ,    // Nord Drum 2    Voice 4   MidiCh 10
            {  57 ,  56 ,  24 ,  23 ,  26 ,  25 ,  10 ,   7 ,      15 ,  17 ,  14 ,  16 ,  18 ,  19 ,  20 ,  21 ,     46 ,  52 ,  53 ,  47 ,  48 ,  54 ,  55 , 254 ,      50 ,  51 ,  49 ,  30 ,  58 , 255 , 255 , 255} ,    // Nord Drum 2    Voice 5   MidiCh 11
            {  57 ,  56 ,  24 ,  23 ,  26 ,  25 ,  10 ,   7 ,      15 ,  17 ,  14 ,  16 ,  18 ,  19 ,  20 ,  21 ,     46 ,  52 ,  53 ,  47 ,  48 ,  54 ,  55 , 254 ,      50 ,  51 ,  49 ,  30 ,  58 , 255 , 255 , 255} ,    // Nord Drum 2    Voice 6   MidiCh 12
            { 255 , 255 , 255 , 255 , 255 , 255 , 255 , 255 ,     255 , 255 , 255 , 255 , 255 , 255 , 255 , 255 ,    255 , 255 , 255 , 255 , 255 , 255 , 255 , 255 ,     255 , 255 , 255 , 255 , 255 , 255 , 255 , 255} ,    //not in Use
            { 255 , 255 , 255 , 255 , 255 , 255 , 255 , 255 ,     255 , 255 , 255 , 255 , 255 , 255 , 255 , 255 ,    255 , 255 , 255 , 255 , 255 , 255 , 255 , 255 ,     255 , 255 , 255 , 255 , 255 , 255 , 255 , 255}};    //not in Use          
       //   {  45 ,  46 ,  47 ,  48 ,  62 ,   5 ,  63 ,  61 ,      18 ,  19 ,  20 ,  21 ,  30 ,  31 ,  58 ,  57 ,     14 ,  15 ,  16 ,  17 ,  28 ,  29 ,  54 ,  55 ,      56 ,  50 ,  51 ,  52 ,  53 , 255 , 255 , 255    ,255 } ,    // Waldorf Pulse1 Page  1   MidiCh  6
       //   {  32 ,  36 , 102 ,  24 , 109 , 112 , 115 ,  33 ,      37 , 103 ,  25 , 108 , 111 , 117 ,  34 ,  38 ,    106 ,  26 , 110 , 113 , 116 ,  35 ,  39 ,  41 ,      40 ,  27 , 255 , 255 , 255 , 255 , 255 , 255    ,255 }};    // Waldorf Pulse1 Page  2   MidiCh  6
       //   { 100 , 101 , 102 , 103 , 104 , 105 , 106 , 107 ,     255 , 255 , 255 , 255 , 255 , 255 , 255 , 255 ,    255 , 255 , 255 , 255 , 255 , 255 , 255 , 255 ,     255 , 255 , 255 , 255 , 255 , 255 , 255 , 255    ,255 } ,    //example Jomox Mbase01 100-107
    //        -----------------------------------------------     -----------------------------------------------    -----------------------------------------------     ---------------------------------------------
    // 57:ClipTyp, 56:ClipLev  24:DistTyp 23:Dist  26-25:EQ      15:N.Filt 17:N.Res  14:N.Freq  19:N.Atk  21:N.Dec   46:Wav 52:Timbre   47:TimDec   54:Bend50:DecHi 51:DecLo 58:Bal 
    // 10:Pan 7:Gain   >>> Example 1st Row Horiz NordDrum2

   //303 when changing mapping, but want to hold the Values! (want to have for example Gain-CC on a other Position     
    const u8 CC_Map_ported[8][32] = { // CC_Map0 [Part]        [Internal CC Nr] = value of external CC   >>
                                // CC_Map1 [MidiChannel] [Remote/Source]  = Value of Synth/Destination
              //  1st Row Horizontal      (top-row)               //  2nd Row Horizontal                             //  3rd Row Horizontal                              //  4th Row Horizontal
 // CC-on-LRE:  0     1     2     3     4     5     6     7         8     9    10    11    12    13    14    15       16    17    18    19    20    21    22    23        24    25    26    27    28    29    30    31  
            {  57 ,  56 ,  24 ,  23 ,  26 ,  25 ,  10 ,   7 ,      15 ,  17 ,  14 ,  16 ,  18 ,  19 ,  20 ,  21 ,     46 ,  52 ,  53 ,  47 ,  48 ,  54 ,  55 , 254 ,      50 ,  51 ,  49 ,  30 ,  58 , 255 , 255 , 255} ,    // Nord Drum 2    Voice 1   MidiCh  7
            {  57 ,  56 ,  24 ,  23 ,  26 ,  25 ,  10 ,   7 ,      15 ,  17 ,  14 ,  16 ,  18 ,  19 ,  20 ,  21 ,     46 ,  52 ,  53 ,  47 ,  48 ,  54 ,  55 , 254 ,      50 ,  51 ,  49 ,  30 ,  58 , 255 , 255 , 255} ,    // Nord Drum 2    Voice 2   MidiCh  8
            {  57 ,  56 ,  24 ,  23 ,  26 ,  25 ,  10 ,   7 ,      15 ,  17 ,  14 ,  16 ,  18 ,  19 ,  20 ,  21 ,     46 ,  52 ,  53 ,  47 ,  48 ,  54 ,  55 , 254 ,      50 ,  51 ,  49 ,  30 ,  58 , 255 , 255 , 255} ,    // Nord Drum 2    Voice 3   MidiCH  9
            {  57 ,  56 ,  24 ,  23 ,  26 ,  25 ,  10 ,   7 ,      15 ,  17 ,  14 ,  16 ,  18 ,  19 ,  20 ,  21 ,     46 ,  52 ,  53 ,  47 ,  48 ,  54 ,  55 , 254 ,      50 ,  51 ,  49 ,  30 ,  58 , 255 , 255 , 255} ,    // Nord Drum 2    Voice 4   MidiCh 10
            {  57 ,  56 ,  24 ,  23 ,  26 ,  25 ,  10 ,   7 ,      15 ,  17 ,  14 ,  16 ,  18 ,  19 ,  20 ,  21 ,     46 ,  52 ,  53 ,  47 ,  48 ,  54 ,  55 , 254 ,      50 ,  51 ,  49 ,  30 ,  58 , 255 , 255 , 255} ,    // Nord Drum 2    Voice 5   MidiCh 11
            {  57 ,  56 ,  24 ,  23 ,  26 ,  25 ,  10 ,   7 ,      15 ,  17 ,  14 ,  16 ,  18 ,  19 ,  20 ,  21 ,     46 ,  52 ,  53 ,  47 ,  48 ,  54 ,  55 , 254 ,      50 ,  51 ,  49 ,  30 ,  58 , 255 , 255 , 255} ,    // Nord Drum 2    Voice 6   MidiCh 12
            { 255 , 255 , 255 , 255 , 255 , 255 , 255 , 255 ,     255 , 255 , 255 , 255 , 255 , 255 , 255 , 255 ,    255 , 255 , 255 , 255 , 255 , 255 , 255 , 255 ,     255 , 255 , 255 , 255 , 255 , 255 , 255 , 255} ,    //not in Use
            { 255 , 255 , 255 , 255 , 255 , 255 , 255 , 255 ,     255 , 255 , 255 , 255 , 255 , 255 , 255 , 255 ,    255 , 255 , 255 , 255 , 255 , 255 , 255 , 255 ,     255 , 255 , 255 , 255 , 255 , 255 , 255 , 255}};    //not in Use          
       //   

       
   // 4 CC_Route_Mode = 1: = 8x Channelstrip          
    const u8 CC_Map1[8][32] = { //CC_Map1 [Active_Strip_Set] [CC to remap to Map0]
              //  1st Row Horizontal                              //  2nd Row Horizontal                             //  3rd Row Horizontal                               //  4th Row Horizontal
            {   1 ,   1 ,   1 ,   1 ,   1 ,   1 ,  32 ,  32 ,      28 ,  28 ,  28 ,  28 ,  28 ,  28 ,  32 ,  32 ,       3 ,   3 ,   3 ,   3 ,   3 ,   3 ,  32 ,  32 ,       7 ,   7 ,   7 ,   7 ,   7 ,   7 ,  32 ,  32  } ,    // Channel-Strip-Set1 (Mixer)
            {   8 ,   8 ,   8 ,   8 ,   8 ,   8 ,  32 ,  32 ,       9 ,   9 ,   9 ,   9 ,   9 ,   9 ,  32 ,  32 ,       4 ,   4 ,   4 ,   4 ,   4 ,   4 ,  32 ,  32 ,       5 ,   5 ,   5 ,   5 ,   5 ,   5 ,  32 ,  32  } ,    // Channel-Strip-Set2 (Filter)
            {  15 ,  15 ,  15 ,  15 ,  15 ,  15 ,  32 ,  32 ,      19 ,  19 ,  19 ,  19 ,  19 ,  19 ,  32 ,  32 ,      24 ,  24 ,  24 ,  24 ,  24 ,  24 ,  32 ,  32 ,      25 ,  25 ,  25 ,  25 ,  25 ,  25 ,  32 ,  32  } ,    // Channel-Strip-Set2 (Decay)
            {  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,      32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,      32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,      32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32  } ,  //not used
            {  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,      32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,      32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,      32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32  } ,  //not used
            {  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,      32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,      32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,      32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32  } ,  //not used
            {  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,      32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,      32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,      32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32  } ,  //not used
            {  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,      32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,      32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,      32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32 ,  32  }};//not used
          // CC_Map1[0][0]=1 > shows CC_Map0[x][1] = Synth-CC:56 = ClickLevel-CC
          
// to get:   CC_Map0[0-7][ CC_Map1[0][0-7] ] = 56CC-Nr.= ClickGain  CC_Map0[0-7][ CC_Map1[0][8-15] ] = 58CC-Nr. = Bal      in the End: CC_Map0[0-7][ CC_Map1[Active_Strip_Set][0-32] ] = ..CC-Nr. 
//                                1                                                    28
// to get:   CC_Map0[0-7][ CC_Map1[1][0-7] ] = 15CC-Nr.= N.Filt     CC_Map0[0-7][ CC_Map1[1][8-15] ] = 17CC-Nr.= N.Res     For Value:  beat[8].CC_Store[32]
//                                8                                                     9                                              beat[0-7].CC_Store[ CC_Map1[Active_Strip_Set][0-32] ]
// to get:   CC_Map0[0-7][ CC_Map1[2][0-7] ] = 21CC-Nr.= N.Decay    CC_Map0[0-7][ CC_Map1[2][8-15] ] = 47CC-Nr.= TimDec
//                                15                                                   19
// to get:   CC_Map0[0-7][ CC_Map1[3][0-7] ] = 255CC-Nr.= Nothing-will be filtered out! > blank out LEDRING
//                                32

          // this shows only Routing, the value is saved via:
          // beat[0-7].CC_Store[0-31];

      
      // POSSIBLE VALUES:
      // * 0-127 CC (normal Midi ControlChange)
      // The Following are Master OFFsets for CCs: what is a Master-Offset? > 
      // * 128: Master Offset for CC0
      // * 129: Master Offset for CC1
      // * 130: Master Offset for CC2
      // * 131: Master Offset for CC3
      // * 254: Tune MSB Pitchbend CC   --- you have to rewrite the code yourself... just surch for 254...
      // * 255: should make always BLANK dont matter what you moove - to indicate there is nothing to control with
      //       we loose the Offset-Mapping for the 128th CC with that!




/*
//CC-Routing-2-Synth MIN MAX Value >>> PRESET >>> Routing! ... for example you dont need full Attack (because it is a bassliner) but want to use alle 16LEDs to show it...
  //Example Waldorf Pulse 2    
  u8  CC_Map0_MinMax[8][32][2] = {
      {0,0,0,0,0,0,0,0,       0,0,0,0,0,0,0,0,       0,0,0,0,0,0,0,0,       0,0,0,0,0,0,0,0}
      {127,127,127,127,127,127,127,127,   127,127,127,127,127,127,127,127,   127,127,127,127,127,,127,127,127,   127,127,127,127,127,127,127,127},
 
      */

     

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Coding only Variables --- No normally user Settings here
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    u8 Active_Strip_Set = 0; //Active Channel Strip SET
    u8 Strip_CH = 0; //last Mooved Encoder makes Channelstrip Nummer...?
    u8 channel = 0; 
    u8 assign = 0; // 0=normal mode, 1=velocity assignment to CCs
    u8 Ring[32] = {};         // hardwareside Ledrings (value dividet by 8 for 16 LEDs...)
    u8 Ring_Style[32] = {};   // blink types, static led and so on

    u8 Velo_From_Note[8] = {0,0,0,0,0,0,0,0};

    u8 MSQ_Clear_Sel   = 0;
    u8 MSQ_Clear_All   = 0;
    u8 VEL_LastMoved   = 0;  // last Moved Encoder in  Velocity Assign Page --- needet to clear Selected Encoder from Velocity Assignment
    
    // Store-Load Variables
    u16 SongNrStor = 0; // store song
    u16 SongNrLoad = 0; // load  song
    u16 PCIN = 0;       // received Programchange Value 


const mios32_enc_config_t encoders[35] = {//(SR begin with 1, ENC with 0) // setup the Pinout of Encoders
    // 1. LRE
  	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=1, .cfg.pos=0 },		//CC-Enc 0
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=1, .cfg.pos=2 },		//CC-Enc 1
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=1, .cfg.pos=4 },      //CC-Enc 2
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=1, .cfg.pos=6 },      //CC-Enc 3
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=2, .cfg.pos=0 },      //CC-Enc 4
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=2, .cfg.pos=2 },      //CC-Enc 5
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=2, .cfg.pos=4 },      //CC-Enc 6	
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=2, .cfg.pos=6 },      //CC-Enc 7
    //row1
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=3, .cfg.pos=0 },      //CC-Enc 8
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=3, .cfg.pos=2 },      //CC-Enc 9
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=3, .cfg.pos=4 },      //CC-Enc 10
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=3, .cfg.pos=6 },      //CC-Enc 11
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=4, .cfg.pos=0 },      //CC-Enc 12
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=4, .cfg.pos=2 },      //CC-Enc 13
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=4, .cfg.pos=4 },      //CC-Enc 14
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=4, .cfg.pos=6 },      //CC-Enc 15
    
    // 2. LRE
  	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=5, .cfg.pos=0 },		//CC-Enc 16
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=5, .cfg.pos=2 },		//CC-Enc 17 
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=5, .cfg.pos=4 },      //CC-Enc 18
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=5, .cfg.pos=6 },      //CC-Enc 19
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=6, .cfg.pos=0 },      //CC-Enc 20
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=6, .cfg.pos=2 },      //CC-Enc 21
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=6, .cfg.pos=4 },      //CC-Enc 22
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=6, .cfg.pos=6 },      //CC-Enc 23
    //row2
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=7, .cfg.pos=0 },      //CC-Enc 24
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=7, .cfg.pos=2 },      //CC-Enc 25
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=7, .cfg.pos=4 },      //CC-Enc 26
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=7, .cfg.pos=6 },      //CC-Enc 27
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=8, .cfg.pos=0 },      //CC-Enc 28
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=8, .cfg.pos=2 },      //CC-Enc 29
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=8, .cfg.pos=4 },      //CC-Enc 30
	{ .cfg.type=DETENTED2, .cfg.speed=FAST, .cfg.speed_par=2, .cfg.sr=8, .cfg.pos=6 },      //CC-Enc 31

    // Extra Encoder
	{ .cfg.type=DETENTED2, .cfg.speed=NORMAL, .cfg.speed_par=4, .cfg.sr=9, .cfg.pos=0 },    //32: MSQ-Loop-Indicator
	{ .cfg.type=DETENTED2, .cfg.speed=NORMAL, .cfg.speed_par=4, .cfg.sr=9, .cfg.pos=2 },    //33: Track Select
	{ .cfg.type=DETENTED2, .cfg.speed=FAST,   .cfg.speed_par=2, .cfg.sr=9, .cfg.pos=4 },    //34: Velo-Morph-Offset
	};



    
void APP_Init(void){         u16 x = 0, y = 0, z = 0;  // for FOR loops
    
    // init all LEDs
    MIOS32_BOARD_LED_Init(0xffffffff);

	// init Encoders
	for(x=0; x<35; ++x) MIOS32_ENC_ConfigSet(x, encoders[x]);

    // init SD-Card
    xSDCardSemaphore 	= xSemaphoreCreateRecursiveMutex(); // create semaphores
    FILE_Init(0);										    // initialize file functions 

    
    /////////////////////////////////////////////////////////////////////////
    // Initate Flags
    flag.PCLoad        =  0;
    flag.PCSave        =  0;
    flag.Tact_Ex_Load  =  0;
    flag.NeedSync      =  1; // to indicate that PAGE length has changed
    flag.FirstTic      =  1;
    flag.Update_Synths  =  0;
    flag.copy          =  0;
    flag.paste         =  0;
    flag.msq_clear_sel =  0;
    flag.msq_clear_all =  0;
    flag.vel_clear_all =  1;
    flag.vel_clear_sel =  0;
    flag.update_synth[0]= 0;
    flag.update_synth[1]= 0;
    flag.update_synth[2]= 0;
    flag.update_synth[3]= 0;
    flag.update_synth[4]= 0;
    flag.update_synth[5]= 0;
    flag.update_synth[6]= 0;
    flag.update_synth[7]= 0;
    
    // Init SD-STORAGE-Struct ////////////////////////////////////////////////////////////////////////////////////
    beat[0].rythm           = 4;    // 4x4=16, 5x3=15 usw... maximal duration per matrix page
    beat[0].PAGE_Steps     = 16;    // calculate from 0 on....so 0-15 are 16 Steps  
    beat[0].MainLoop       = 32;    // max Length of the sync_mainloop, which resyncs individual channels
    beat[0].bpm           = 100;

    for(x=0; x<8; x++){for(y=0; y<32; y++){for(z=0; z<256; z++){
        beat[x].MSQ_Store[y][z] =  0;   // [Parts][CCs][Step>Value]
        }}} 
        
    for(x=0; x<8; x++){for(y=0; y<32; y++){
        beat[x].CC_Store[y] = 0;
        beat[x].Velo_Morph[y]   = 63;   // middle postion 0 64=+1  63=-1
        beat[x].CC_Morph[y]     = 63;   // middle postion 0 64=+1  63=-1

        } } // [Parts][CCs]  
                                            
    for(x=0; x < 8; x++) {
        beat[x].loop_lenght_set   = 1; // 0-15 = Value of LED of a ring
        beat[x].MSQ_Active        = 1;
        beat[x].Velo_Morph_Offset = 63;
        beat[x].CC_Morph_Offset   = 63;
        beat[x].MSQ_Rec           = 0;
        beat[x].MSQ_Rec_mem       = 0;   // to save and tomparary overide MSQ_REC

        beat[x].MSQ_LastMoved     = 0;
    }

    for( x=0; x<8; x++ ) {for(y=0; y<32; y++) {for(z=0; z<32; z++) { beat[x].Mod_Matrix[y][z] = 0; }}}
    //
    ///////////////////////////////////////////////////////////////////////////////////////////////////////


    rythm_count = 1; // 1= 4/4 tact
    Activ_CH  = 0;

    
    // Clear Motion Sequences
    for(z=0; z<8;   z++) {
    for(x=0; x<32;  x++) {
    for(y=0; y<256; y++) { beat[z].MSQ_Store[x][y] = 128; }}} //[Parts][CCs][Step>Value]   128 means that it will be ignored

    // SEQUENCER
    MIOS32_MIDI_DirectRxCallback_Init(NOTIFY_MIDI_Rx);
    xTaskCreate(TASK_SEQ, (signed portCHAR *)"SEQ", configMINIMAL_STACK_SIZE, NULL, PRIORITY_TASK_SEQ, NULL);   // install task
    SEQ_Init(0); 
         
    xTaskCreate(TASK_FLAG, (signed portCHAR *)"FLAG", configMINIMAL_STACK_SIZE, NULL, PRIORITY_TASK_FLAG, NULL);   // my own tasks jobs to do
}


void APP_Background(void){    // lo-Priority 

    // Calculate channel
    if(CC_Map_Mode == 0) { channel = Activ_CH; }
    else                 { channel = Strip_CH; }

    
    ///////////////////////////////////////////////////////////////////////////////////////////7        
    static u32 SDCardCount = 0;
    SDCardCount++; //  send Check-SDCard-Commant to StoreLoad-Function (track, clip, job)
    if (SDCardCount > SD_Check_Time) { SDCardCount = 0; SD(0, 3); }   
    ////////////////////////////////////////////////////////////////////////////////////////////



// 32x  L E D    R I N G    C A L C U L A T I O N    
    u8 temp[32]; u8 temp_style[32];   u8 x=0; u8 y=0; u8 value=0;

if (CC_Map_Mode == 0) {

    //////////////////////////////////////////         
    if (assign == 0) { // MSQ + NORMAL MODE
            for(x=0; x<32; x++) {       // count thru all HardWare LEDRINGS

            // MSQ-Value - Calculation
            value = beat[channel].MSQ_Store[x][loop_step_pos[channel]];

            if   ( value != 128  &&  beat[channel].MSQ_Active == 1) { temp_style[x] = blink_64;} // blink LED
            else { value = beat[channel].CC_Store[x];                 temp_style[x] = 1; }       // static activated LED
            temp[x] = value >> 3;
            }
    }

    //////////////////////////////////////////         
    if (assign == 1) { // Velocity Assign Mode
            for(x=0; x<32; x++) {       // count thru all HardWare LEDRINGS
            
            // Assign-Value - Calculation
            value = beat[channel].Velo_Morph[x];

            if (   value == 63) { temp_style[x]  = 128; } // = no Velocity Assign Effect! No LED! DEACTIVATE!
            else { temp_style[x] = blink_mod; } // blink activate LED

            temp[x] = value >> 3;
            }
    }
}


if ( CC_Map_Mode == 1 ) {   // Global Mode      //  Channel Strips  //  CC-Overview of importend Parameters  

    /////////////////////////////////////////////
    if (assign == 0) { // MSQ + CC Storage
            for(x=0; x<32; x++) {       // count thru all HardWare LEDRINGS
            
            // Motion-SEQ-Value - Calculation
            value = beat[y].MSQ_Store[ CC_Map1[Active_Strip_Set][x] ][loop_step_pos[y]];
            
                if (value != 128  &&  beat[y].MSQ_Active == 1 ) {temp_style[x] = blink_64;} 
            
              else  {  // "Static" CC-Storage  (satic in sense of not automated)
               //  0-31         0-7              0-32     0-7        0-32 
               // HW-CC  = storage[Ch]     [  remap [Active Set][Nr]  ]
                value  = beat[y].CC_Store[ CC_Map1[Active_Strip_Set][x] ]; temp_style[x] = 1;}

            temp[x] = value >> 3;
            
                y++;                    // count thru Synth-Voices/Tracks
                if (y > 7) { y = 0;}    // next Controll next Row, begin from start counting 0-7 Voices
            } }

    //////////////////////////////////////////         
    if (assign == 1) { // Velocity Assign Mode
            for(x=0; x<32; x++) {       // count thru all HardWare LEDRINGS
            
            // Assign-Value - Calculation
            value = beat[y].Velo_Morph[ CC_Map1[Active_Strip_Set][x] ];

            if ( value == 63) { temp_style[x]  = 128; } // = no Velocity Assign Effect! No LED! DEACTIVATE!
            else { temp_style[x] = 1; } // static activate LED

                
            temp[x]  = beat[y].Velo_Morph[ CC_Map1[Active_Strip_Set][x] ] >> 3;
                        
                y++;                    // count thru Synth-Voices/Tracks
                if (y > 7) { y = 0;}    // next Controll next Row, begin from start counting 0-7 Voices
            }
    }
} // MAP Mode 1



    // LRE Conversion Table
    Ring[0]   = temp[0];
    Ring[2]   = temp[1];
    Ring[4]   = temp[2];
    Ring[6]   = temp[3];
    Ring[8]   = temp[4];
    Ring[10]  = temp[5];
    Ring[12]  = temp[6];
    Ring[14]  = temp[7];

    Ring[1]   = temp[8];    
    Ring[3]   = temp[9];    
    Ring[5]   = temp[10];    
    Ring[7]   = temp[11];    
    Ring[9]   = temp[12];    
    Ring[11]  = temp[13];    
    Ring[13]  = temp[14];    
    Ring[15]  = temp[15];    

    Ring[16]  = temp[16];    
    Ring[18]  = temp[17];    
    Ring[20]  = temp[18];    
    Ring[22]  = temp[19];    
    Ring[24]  = temp[20];    
    Ring[26]  = temp[21];    
    Ring[28]  = temp[22];    
    Ring[30]  = temp[23];
               
    Ring[17]  = temp[24];    
    Ring[19]  = temp[25];    
    Ring[21]  = temp[26];    
    Ring[23]  = temp[27];    
    Ring[25]  = temp[28];    
    Ring[27]  = temp[29];    
    Ring[29]  = temp[30];    
    Ring[31]  = temp[31];


    Ring_Style[0]   = temp_style[0];
    Ring_Style[2]   = temp_style[1];
    Ring_Style[4]   = temp_style[2];
    Ring_Style[6]   = temp_style[3];
    Ring_Style[8]   = temp_style[4];
    Ring_Style[10]  = temp_style[5];
    Ring_Style[12]  = temp_style[6];
    Ring_Style[14]  = temp_style[7];

    Ring_Style[1]   = temp_style[8];    
    Ring_Style[3]   = temp_style[9];    
    Ring_Style[5]   = temp_style[10];    
    Ring_Style[7]   = temp_style[11];    
    Ring_Style[9]   = temp_style[12];    
    Ring_Style[11]  = temp_style[13];    
    Ring_Style[13]  = temp_style[14];    
    Ring_Style[15]  = temp_style[15];    

    Ring_Style[16]  = temp_style[16];    
    Ring_Style[18]  = temp_style[17];    
    Ring_Style[20]  = temp_style[18];    
    Ring_Style[22]  = temp_style[19];    
    Ring_Style[24]  = temp_style[20];    
    Ring_Style[26]  = temp_style[21];    
    Ring_Style[28]  = temp_style[22];    
    Ring_Style[30]  = temp_style[23];
               
    Ring_Style[17]  = temp_style[24];    
    Ring_Style[19]  = temp_style[25];    
    Ring_Style[21]  = temp_style[26];    
    Ring_Style[23]  = temp_style[27];    
    Ring_Style[25]  = temp_style[28];    
    Ring_Style[27]  = temp_style[29];    
    Ring_Style[29]  = temp_style[30];    
    Ring_Style[31]  = temp_style[31];

    flag.Seq_is_running = SEQ_BPM_IsRunning(); // get seq-running status, and save it into variable

}


static void TASK_FLAG(void *pvParameters) { //  die vorhandenen aufgaben hier sinnvoll?
    
  portTickType xLastExecutionTime;

  // Initialise the xLastExecutionTime variable on task entry
  xLastExecutionTime = xTaskGetTickCount();

  while( 1 ) {
    vTaskDelayUntil(&xLastExecutionTime, 1 / portTICK_RATE_MS); // 505 welche rate für diese operation?

    // Copy Paste Single Drum-voices
    if(flag.copy  == 1) { flag.copy  = 0; memcpy ( &beat_cpy,  &beat[Activ_CH],  sizeof (store_t) ); } // COPY

    if(flag.paste == 1) { flag.paste = 0; memcpy ( &beat[Activ_CH],  &beat_cpy,  sizeof (store_t) );
                          flag.update_synth[Activ_CH] = 1; flag.NeedSync = 1;}  // PASTE

    if(flag.msq_clear_sel  == 1) { flag.msq_clear_sel = 0; u16 x;
            if(CC_Map_Mode == 0) {  for(x=0; x<256; x++) { beat[Activ_CH].MSQ_Store[beat[Activ_CH].MSQ_LastMoved][x]=128; }}
            if(CC_Map_Mode == 1) {  for(x=0; x<256; x++) { beat[Strip_CH].MSQ_Store[beat[Strip_CH].MSQ_LastMoved][x]=128; }}  }
                
    if(flag.msq_clear_all == 1) { flag.msq_clear_all = 0; u16 x,y;
            if(CC_Map_Mode == 0) {  for(x=0; x<256; x++) { for(y=0; y<32; y++) {beat[Activ_CH].MSQ_Store[y][x]=128;}}} // Deep Edit
            if(CC_Map_Mode == 1) {  for(x=0; x<256; x++) { for(y=0; y<32; y++) {beat[Strip_CH].MSQ_Store[y][x]=128;}}} } // Channel Strip

    if(flag.vel_clear_sel == 1) { flag.vel_clear_sel = 0; beat[Activ_CH].Velo_Morph[VEL_LastMoved] = 63;} // set ALL Morph 0
    
    if(flag.vel_clear_all == 1) { flag.vel_clear_all = 0; u16 y; for(y=0; y<32; y++) {beat[Activ_CH].Velo_Morph[y] = 63;}} // set ALL Morph 0


    if (  flag.Update_Synths == 1 ) {
          static u16 DUMP_ALL = 0;
          
            DUMP_ALL++;
            if( DUMP_ALL >= DumpOutRateSynth)   {
            
                            DUMP_ALL = 0; // reset counter, for next cycle
                            
                            static u8 x = 0; 
                            x++; //dump CC number out 
                            
                            if(x > 31){ x = 0; flag.Update_Synths = 0;} // when all CCs are sent out, delete the Flag...
            
                                static u8 track = 0; 
                                for(track=0; track<8; track++) {

                                u8 cc    = CC_Map0[track][x];
                                u8 value = beat[track].CC_Store[x];
        
                                if(cc  < 128) { MIOS32_MIDI_SendCC(MeloPort, MeloCh[track],   cc,  value); }
                                if(cc == 254) { MIOS32_MIDI_SendCC(MeloPort, MeloCh[track],   63,  63); //finetune fixed
                                                MIOS32_MIDI_SendCC(MeloPort, MeloCh[track],   31,  value);} // tune
                            }}}


    if (  flag.update_synth[Activ_CH] == 1 ) {
          static u16 DUMP = 0;

            DUMP++;
            if( DUMP >= (DumpOutRateSynth / 2) )   {

                                DUMP = 0; // reset counter, for next cycle
                                
                                static u8 x = 0; 
                                x++; //dump CC number out 
                                
                                if(x > 31){ x = 0; flag.update_synth[Activ_CH] = 0;} // when all CCs are sent out, delete the Flag...
                                
                                    u8 cc    = CC_Map0[Activ_CH][x];
                                    u8 value = beat[Activ_CH].CC_Store[x];
            
                                    if(cc  < 128) { MIOS32_MIDI_SendCC(MeloPort, MeloCh[Activ_CH],   cc,  value); }
                                    if(cc == 254) { MIOS32_MIDI_SendCC(MeloPort, MeloCh[Activ_CH],   63,  63); //finetune fixed
                                                    MIOS32_MIDI_SendCC(MeloPort, MeloCh[Activ_CH],   31,  value);} // tune
                                }}
}
}//End Hook


void APP_Tick(void){ // some Timers
        // Programchange Load when Stopped turning the Wheel (from External Source)
    if( flag.PCLoad == 1 )  {
            static u32 x = 0;
            x++; 
            if( x > PC_Load_Timer ) {   x            = 0;       // init counter for the next run
                                        flag.PCLoad  = 0;       // Kill Flag to avoid double triggering events
                                        SD(SongNrLoad, 1); }}   // LOAD

    if( flag.PCSave == 1 )  {
            static u32 x = 0;
            x++;

            if( x > PC_Save_Timer ) {   x            = 0;     // init counter for the next run
                                        flag.PCSave  = 0;     // Kill Flag to avoid double triggering events
                                        SD(SongNrStor, 2);
                                        }} // Store

    if( flag.Tact_Ex_Load == 1 )  {
            static u32 x = 0;
            x++; 
            if( x >  PC_Load_Timer  + 10 ) {
                                        x                  = 0;         // init counter for the next run
                                        flag.Tact_Ex_Load  = 0;         // Kill Flag to avoid double triggering events
                                        beat[0].rythm = CC_Tact_Ext_Storage;   // external received Tact over-ruling internal saved Tact
                    switch(CC_Tact_Ext_Storage) {
                        case 3:  rythm_count = 1; beat[0].PAGE_Steps = 12; flag.NeedSync = 1; beat[0].MainLoop = beat[0].rythm * 8; break;
                        case 4:  rythm_count = 2; beat[0].PAGE_Steps = 16; flag.NeedSync = 1; beat[0].MainLoop = beat[0].rythm * 8; break; 
                        case 5:  rythm_count = 3; beat[0].PAGE_Steps = 10; flag.NeedSync = 1; beat[0].MainLoop = beat[0].rythm * 8; break; 
                        case 7:  rythm_count = 5; beat[0].PAGE_Steps = 14; flag.NeedSync = 1; beat[0].MainLoop = beat[0].rythm * 8; break; 
                        case 9:  rythm_count = 6; beat[0].PAGE_Steps =  9; flag.NeedSync = 1; beat[0].MainLoop = beat[0].rythm * 8; break; 
                        case 11: rythm_count = 7; beat[0].PAGE_Steps = 11; flag.NeedSync = 1; beat[0].MainLoop = beat[0].rythm * 8; break; 
                        case 13: rythm_count = 8; beat[0].PAGE_Steps = 13; flag.NeedSync = 1; beat[0].MainLoop = beat[0].rythm * 8; break;
                    }}} // Done
                                        

    } // called each mS from main task which also handles DIN, ENC and AIN events. You could add more jobs here - dont do more than 300uS!

       
void APP_MIDI_NotifyPackage(mios32_midi_port_t port, mios32_midi_package_t midi_package){
    
// local Variables
static u8 Bank = 0; // CC  for  Program-Change Bank

    // Midi CCs
    switch(port) {
        
        case MeloPort: /////////////////////////////////////////////////////////////////////////////////////////////////////////

                // forwarding Midiclock & Transport     //    248CLK,250Strt,251Cont,252Stp
                if( TrigMidiThruClock==1  &&  (midi_package.evnt0==248 || midi_package.evnt0==250 || midi_package.evnt0==251 || midi_package.evnt0==252) )
                        { MIOS32_MIDI_SendPackage(MeloPort,  midi_package); break; }


                if( midi_package.chn == PCChIN  &&  midi_package.type == CC) {
                    
                            switch( midi_package.cc_number ) {
                                         
                                        case CC_PC_Bank:    Bank = midi_package.value * 128;  break; // Bank0:0-127, Bank1:1-255...
                                        case CC_Tact_mono:  CC_Tact_Ext_Storage = midi_package.value;  flag.Tact_Ex_Load = 1;  break;  // set Rythm
                                        case CC_Save_mono:  SongNrLoad  = midi_package.value + Bank; // generate ProgramChange message via SAVE dialog
                                                            SongNrStor  = SongNrLoad;   
                                                            flag.PCSave = 1;  break; // Save after a while
                            }}
                                    
                            switch(midi_package.evnt0) {
                                    //  Midi CH         PC-Channel         Song-Nr   =              PC-Nr + Bank   Load after a while    also set Store-Number 
                                        case 192:   if( PCChIN == 0 )   { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                                        case 193:   if( PCChIN == 1 )   { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                                        case 194:   if( PCChIN == 2 )   { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                                        case 195:   if( PCChIN == 3 )   { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                                        case 196:   if( PCChIN == 4 )   { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                                        case 197:   if( PCChIN == 5 )   { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                                        case 198:   if( PCChIN == 6 )   { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                                        case 199:   if( PCChIN == 7 )   { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                                        case 200:   if( PCChIN == 8 )   { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                                        case 201:   if( PCChIN == 9 )   { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                                        case 202:   if( PCChIN == 10 )  { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                                        case 203:   if( PCChIN == 11 )  { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                                        case 204:   if( PCChIN == 12 )  { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                                        case 205:   if( PCChIN == 13 )  { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                                        case 206:   if( PCChIN == 14 )  { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                                        case 207:   if( PCChIN == 15 )  { SongNrLoad = midi_package.evnt1 + Bank;  flag.PCLoad = 1;  SongNrStor = SongNrLoad;  break; }
                            }
                            

              // Make Velocity
              if (    midi_package.type == NoteOn  &&  midi_package.velocity > 0 ) {
                    if     ( midi_package.chn == MeloCh[0]  )  { Velo_From_Note[0] = midi_package.velocity; }
                    else if( midi_package.chn == MeloCh[1]  )  { Velo_From_Note[1] = midi_package.velocity; }
                    else if( midi_package.chn == MeloCh[2]  )  { Velo_From_Note[2] = midi_package.velocity; }  
                    else if( midi_package.chn == MeloCh[3]  )  { Velo_From_Note[3] = midi_package.velocity; }  
                    else if( midi_package.chn == MeloCh[4]  )  { Velo_From_Note[4] = midi_package.velocity; }  
                    else if( midi_package.chn == MeloCh[5]  )  { Velo_From_Note[5] = midi_package.velocity; }
                    else if( midi_package.chn == MeloCh[6]  )  { Velo_From_Note[6] = midi_package.velocity; }  
                    else if( midi_package.chn == MeloCh[7]  )  { Velo_From_Note[7] = midi_package.velocity; }
                }
                

              //forwarding Notes
              if(midi_package.evnt0==144) {//Note @ Midi Channel 1
                     if(MeloCh[0]==0)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==0)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==0)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[3]==0)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==0)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==0)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==0)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==0)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} }
         else if(midi_package.evnt0==145) {//Note @ Midi Channel 2
                     if(MeloCh[0]==1)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==1)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==1)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[3]==1)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==1)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==1)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==1)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==1)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} }
         else if(midi_package.evnt0==146) {//Note @ Midi Channel 3
                     if(MeloCh[0]==2)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==2)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==2)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[3]==2)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==2)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==2)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==2)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==2)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} }
         else if(midi_package.evnt0==147) {//Note @ Midi Channel 4
                     if(MeloCh[0]==3)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==3)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==3)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[3]==3)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==3)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==3)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==3)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==3)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} }
         else if(midi_package.evnt0==148) {//Note @ Midi Channel 5
                     if(MeloCh[0]==4)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==4)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==4)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[3]==4)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==4)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==4)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==4)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==4)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} }
         else if(midi_package.evnt0==149) {//Note @ Midi Channel 6
                     if(MeloCh[0]==5)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==5)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==5)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[3]==5)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==5)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==5)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==5)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==5)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} }
         else if(midi_package.evnt0==150) {//Note @ Midi Channel 7
                     if(MeloCh[0]==6)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==6)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==6)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[3]==6)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==6)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==6)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==6)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==6)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} }
         else if(midi_package.evnt0==151) {//Note @ Midi Channel 8
                     if(MeloCh[0]==7)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==7)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==7)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[3]==7)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==7)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==7)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==7)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==7)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} }
         else if(midi_package.evnt0==152) {//Note @ Midi Channel 9
                     if(MeloCh[0]==8)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==8)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==8)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[3]==8)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==8)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==8)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==8)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==8)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} }
         else if(midi_package.evnt0==153) {//Note @ Midi Channel 10
                     if(MeloCh[0]==9)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==9)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==9)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[3]==9)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==9)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==9)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==9)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==9)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} }
         else if(midi_package.evnt0==154) {//Note @ Midi Channel 11
                     if(MeloCh[0]==10)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==10)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==10)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}   
                else if(MeloCh[3]==10)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==10)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==10)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==10)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==10)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} }
         else if(midi_package.evnt0==155) {//Note @ Midi Channel 12
                     if(MeloCh[0]==11)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==11)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==11)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[3]==11)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==11)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==11)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==11)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==11)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} }
         else if(midi_package.evnt0==156) {//Note @ Midi Channel 13
                     if(MeloCh[0]==12)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==12)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==12)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[3]==12)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==12)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==12)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==12)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==12)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} }
         else if(midi_package.evnt0==157) {//Note @ Midi Channel 14
                     if(MeloCh[0]==13)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==13)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==13)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[3]==13)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==13)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==13)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==13)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==13)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} }
         else if(midi_package.evnt0==158) {//Note @ Midi Channel 15
                     if(MeloCh[0]==14)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==14)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==14)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[3]==14)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==14)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==14)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==14)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==14)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} }
         else if(midi_package.evnt0==159) {//Note @ Midi Channel 16
                     if(MeloCh[0]==15)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);}
                else if(MeloCh[1]==15)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[2]==15)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[3]==15)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[4]==15)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[5]==15)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[6]==15)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} 
                else if(MeloCh[7]==15)   {MIOS32_MIDI_SendPackage(MeloPort,  midi_package);} } 
        }
        }


void APP_SRIO_ServicePrepare(void){
    
    u8 ring_sys[3]; u8 ring_sys_style[3];

    /////////////////////////////////////////////////
    // Single L E D
    
    // MSQ-REC  -  LED    
    if(flag.Seq_is_running == 0)                                      {  MIOS32_DOUT_PinSet(82, beat[channel].MSQ_Rec); }
    else if(flag.Seq_is_running == 1  &&  beat[channel].MSQ_Rec == 0) {  MIOS32_DOUT_PinSet(82, 0); } // 0 is activated
    else                                                              {  MIOS32_DOUT_PinSet(82, blink_32); } // blinking with seq tact
    // MSQ-Active  &   MSQ-REC  share the same LED
    if(beat[channel].MSQ_Active == 1  && beat[channel].MSQ_Rec == 0  &&  assign == 0) {   MIOS32_DOUT_PinSet(82, blink_main_seq); }

    // Map Mode  LED
    MIOS32_DOUT_PinSet(83, CC_Map_Mode);

    /////////////////////////////////////////////////
    // System Ledrings
    switch(CC_Map_Mode) {
        
            case 0: ring_sys[0]  = 15  - beat[channel].loop_lenght_set;   ring_sys_style[0] = 1;  // 15- inverts value
                    ring_sys[1]  = 14  - Activ_CH;   ring_sys_style[1] = 1;
                    ring_sys[2]  = (127 - beat[channel].Velo_Morph_Offset) >> 3;   ring_sys_style[2] = 1;
                    break;
                    
            case 1: ring_sys[0]  = 15  - beat[channel].loop_lenght_set;   ring_sys_style[0] = 1;  // 15- inverts value
                    ring_sys[1]  = 14  - Active_Strip_Set;   ring_sys_style[1] = 1;
                    ring_sys[2]  = (127 - beat[channel].Velo_Morph_Offset) >> 3;   ring_sys_style[2] = 1;
                    break;}
            
    //1. Deactivate all Leds
    MIOS32_DOUT_PinSet( 64, 0); // LED 0
    MIOS32_DOUT_PinSet( 65, 0); // LED 1 
    MIOS32_DOUT_PinSet( 66, 0); // LED 2
    MIOS32_DOUT_PinSet( 67, 0); // LED 3
    MIOS32_DOUT_PinSet( 68, 0); // LED 4
    MIOS32_DOUT_PinSet( 69, 0); // LED 5
    MIOS32_DOUT_PinSet( 70, 0); // LED 6
    MIOS32_DOUT_PinSet( 71, 0); // LED 7
    MIOS32_DOUT_PinSet( 72, 0); // LED 8
    MIOS32_DOUT_PinSet( 73, 0); // LED 9
    MIOS32_DOUT_PinSet( 74, 0); // LED 10 
    MIOS32_DOUT_PinSet( 75, 0); // LED 11 
    MIOS32_DOUT_PinSet( 76, 0); // LED 12 
    MIOS32_DOUT_PinSet( 77, 0); // LED 13 
    MIOS32_DOUT_PinSet( 78, 0); // LED 14     - here 0 is OFF
    
    MIOS32_DOUT_PinSet( 79, 1); // Common 0 
    MIOS32_DOUT_PinSet( 80, 1); // Common 1
    MIOS32_DOUT_PinSet( 81, 1); // Common 2   -  here 1 is OFF

    
    //2. Activate Common PIN
    //count thru the Ring-Common-Pins
    static s8  leds_h = 0;
    leds_h++; if (leds_h > 2) {leds_h = 0;}

    switch(leds_h) {
        case 0: MIOS32_DOUT_PinSet(79,  0);  break;
        case 1: MIOS32_DOUT_PinSet(80,  0);  break;            
        case 2: MIOS32_DOUT_PinSet(81,  0);  break;            
    }

    // 3. Activate Vertical Lines  
    MIOS32_DOUT_PinSet( 64 + ring_sys[leds_h],      ring_sys_style[leds_h]);

    // MSQ-Position-INDICATOR
    if (leds_h == 0) { MIOS32_DOUT_PinSet( 64 + (14-IND), blink_32); } //only on 1st Encoder, only in Track-Mode



    // 1. Deactivate Vertical PINs (from last cycle)
    // 1.LRE
    MIOS32_DOUT_PinSet( 16, 0); 
    MIOS32_DOUT_PinSet( 17, 0); 
    MIOS32_DOUT_PinSet( 18, 0);
    MIOS32_DOUT_PinSet( 19, 0); 
    MIOS32_DOUT_PinSet( 20, 0); 
    MIOS32_DOUT_PinSet( 21, 0); 
    MIOS32_DOUT_PinSet( 22, 0); 
    MIOS32_DOUT_PinSet( 23, 0); 
    MIOS32_DOUT_PinSet( 24, 0); 
    MIOS32_DOUT_PinSet( 25, 0); 
    MIOS32_DOUT_PinSet( 26, 0); 
    MIOS32_DOUT_PinSet( 27, 0); 
    MIOS32_DOUT_PinSet( 28, 0); 
    MIOS32_DOUT_PinSet( 29, 0); 
    MIOS32_DOUT_PinSet( 30, 0); 
    MIOS32_DOUT_PinSet( 31, 0); 
    // 2.LRE
    MIOS32_DOUT_PinSet( 48, 0); 
    MIOS32_DOUT_PinSet( 49, 0); 
    MIOS32_DOUT_PinSet( 50, 0); 
    MIOS32_DOUT_PinSet( 51, 0);
    MIOS32_DOUT_PinSet( 52, 0); 
    MIOS32_DOUT_PinSet( 53, 0); 
    MIOS32_DOUT_PinSet( 54, 0); 
    MIOS32_DOUT_PinSet( 55, 0); 
    MIOS32_DOUT_PinSet( 56, 0); 
    MIOS32_DOUT_PinSet( 57, 0); 
    MIOS32_DOUT_PinSet( 58, 0); 
    MIOS32_DOUT_PinSet( 59, 0); 
    MIOS32_DOUT_PinSet( 60, 0); 
    MIOS32_DOUT_PinSet( 61, 0); 
    MIOS32_DOUT_PinSet( 62, 0); 
    MIOS32_DOUT_PinSet( 63, 0);
    
    /////DOUT-Matrix-routine/////
    static s8  LEDCount   = 0;      // only to calculate
    static s8  LEDCount_H = 0;      // only to calculate
    
    // 2. Deactivate the last Horizontal PIN (from last cycle)
    // 1.LRE
    MIOS32_DOUT_PinSet(LEDCount_H,      0);    // 1.LRE
    MIOS32_DOUT_PinSet(LEDCount_H + 32, 0);    // 2.LRE   
    LEDCount_H =  LEDCount;               // LEDDoutH = Number of Connected Register*8=Pin Number

    
    // 3. Activate Vertical Lines
        if( Ring_Style[LEDCount]    != 128 )   { // switched OFF? dont want a LED    128: turn off/ do nothing
            MIOS32_DOUT_PinSet( 16 + Ring[LEDCount],      Ring_Style[LEDCount]);  }     // 1.LRE
        if( Ring_Style[LEDCount+16] != 128 )   { // switched OFF? 
            MIOS32_DOUT_PinSet( 48 + Ring[LEDCount + 16], Ring_Style[LEDCount + 16]); } // 2.LRE
            

    // 4. Activate Horizontal Line 
    MIOS32_DOUT_PinSet(LEDCount_H,      1); //Activate First Row - Negatives Side of Diodes   -  1.LRE
    MIOS32_DOUT_PinSet(LEDCount_H + 32, 1); //Activate First Row - Negatives Side of Diodes   -  2.LRE


    // 5. Counter - count from 0-15 then 0 again...
    if (LEDCount < 16) {LEDCount = LEDCount + 1;}
    if (LEDCount > 15) {LEDCount = 0;}    
	}


void APP_SRIO_ServiceFinish(void){}


void APP_DIN_NotifyToggle(u32 pin, u32 pin_value){
//    MIOS32_MIDI_SendDebugMessage("din pin:%d, value.%d", pin, pin_value);  // DEBUG
    
    switch (pin) {
        // MSQ
        case 70:   if(pin_value == 0) { u8 state = beat[channel].MSQ_Rec; // Save Actual Button State
                                        u8 x; for (x=0; x<8; x++) { beat[x].MSQ_Rec = 0; } // Clear All Recordings
                                        beat[channel].MSQ_Rec =! state; break; }// Toggle Old-State Button
                break;
        case 74:   MSQ_Clear_Sel =! pin_value;    break;  // MSQ Clear Selected  Rotary Encoder
        case 75:   if(pin_value == 0) { flag.msq_clear_all = 1; break; } break; // MSQ Clear ALL Rotary Encoders
        case 76:   if(pin_value == 0) { flag.copy  = 1; break; } break;
        case 77:   if(pin_value == 0) { flag.paste = 1; break; } break;
        
        // Clear ALL Velo-Morphs   -  -  initalize to 63 - which is neutral (-63+)
        case 78:   if(pin_value == 0) { flag.vel_clear_sel = 1; break; }  break;
        case 79:   if(pin_value == 0) { flag.vel_clear_all = 1; break; }  break;

        case 71:   if(pin_value == 0) { CC_Map_Mode =! CC_Map_Mode; break; }  break; // Channel-Strip
        
        case 72:   assign =! pin_value; break; // Channel-Strip
        case 81:   if(pin_value == 0) { flag.PCLoad = 1; break; } break; // Load/Reload Patch
        case 80:   if(pin_value == 0) { flag.PCSave = 1; break; } break; // Store Patch
        
        break;
    }
}

void APP_ENC_NotifyChange(u32 encoder, s32 incrementer) { signed cc;

// //beat[control-8].MSQ_Active =! beat[control-8].MSQ_Active;  // 505 Motion-Sequence Pause implementieren eventuel mit clear select verbinden
// hold clear select and turn msq-encoder -switches it off for example...
    
// MIOS32_MIDI_SendDebugMessage("enc:%d, inc:%d", encoder, incrementer);  // DEBUG

    // Encoders default settings - if no other specified below....
    u8 last_enc = 0;
    u8 cc_max_value = 127;  // as default - most of Encoders Maximal Value is  full  int16 wordlength
    u8 cc_min_value = 0;    // as default - 0 nothing may can make some noise on dac... so better 1 here


    // Remap Encoders  -  Virtual Encoders
    switch( encoder ) {
        
        // LRE Boards
        // Row 0
        case 0:  last_enc =   0; Strip_CH = 0;  break;   // case:           LRE-Enc-NR
        case 2:  last_enc =   1; Strip_CH = 1;  break;   // last_enc:       better routable CC-Nr 
        case 4:  last_enc =   2; Strip_CH = 2;  break;   // Strip_CH:       for channel-Strip Mode
        case 6:  last_enc =   3; Strip_CH = 3;  break;
        case 8:  last_enc =   4; Strip_CH = 4;  break;
        case 10: last_enc =   5; Strip_CH = 5;  break;
        case 12: last_enc =   6; Strip_CH = 6;  break;
        case 14: last_enc =   7; Strip_CH = 7;  break;
        // Row 1
        case 1:  last_enc =   8; Strip_CH = 0;  break;
        case 3:  last_enc =   9; Strip_CH = 1;  break;
        case 5:  last_enc =  10; Strip_CH = 2;  break;
        case 7:  last_enc =  11; Strip_CH = 3;  break;
        case 9:  last_enc =  12; Strip_CH = 4;  break;
        case 11: last_enc =  13; Strip_CH = 5;  break;
        case 13: last_enc =  14; Strip_CH = 6;  break;
        case 15: last_enc =  15; Strip_CH = 7;  break;
        // Row 2
        case 16: last_enc =  16; Strip_CH = 0;  break;
        case 18: last_enc =  17; Strip_CH = 1;  break;
        case 20: last_enc =  18; Strip_CH = 2;  break; 
        case 22: last_enc =  19; Strip_CH = 3;  break; 
        case 24: last_enc =  20; Strip_CH = 4;  break; 
        case 26: last_enc =  21; Strip_CH = 5;  break; 
        case 28: last_enc =  22; Strip_CH = 6;  break; 
        case 30: last_enc =  23; Strip_CH = 7;  break;
        // Row 3
        case 17: last_enc =  24; Strip_CH = 0;  break;
        case 19: last_enc =  25; Strip_CH = 1;  break;
        case 21: last_enc =  26; Strip_CH = 2;  break;
        case 23: last_enc =  27; Strip_CH = 3;  break;
        case 25: last_enc =  28; Strip_CH = 4;  break;
        case 27: last_enc =  29; Strip_CH = 5;  break;
        case 29: last_enc =  30; Strip_CH = 6;  break;
        case 31: last_enc =  31; Strip_CH = 7;  break;
        
        // Settings (extra Board
        case 32: last_enc =  32; cc_min_value = 1; cc_max_value = 15;  break; // MSQ-Loop-Length
        case 33: last_enc =  33; cc_min_value = 0; cc_max_value = 7;   break; // Track Select - Active Part
        case 34: last_enc =  34; cc_min_value = 1; cc_max_value = 127; break; // Velo Morph Offset
    }

    if(CC_Map_Mode == 0) { channel = Activ_CH; }
    if(CC_Map_Mode == 1) { channel = Strip_CH; }
    
            
    // Loop Length Set
    if ( last_enc == 32  &&  assign == 0 )  {
            cc = beat[channel].loop_lenght_set - incrementer;
            if ( cc <  cc_min_value ) { cc =  cc_min_value; }  // fix minimal value
            if ( cc >  cc_max_value ) { cc =  cc_max_value; }  // fix maximalvalue
            beat[channel].loop_lenght_set = cc;
            flag.NeedSync = 1; } // Set Loop Length when reaching next "0 Main-Loop"-Step
            
    // Rotary Global Morph Offset
    if ( last_enc == 34 ) {
            cc = beat[channel].Velo_Morph_Offset - incrementer;
            if ( cc <=  cc_min_value ) { cc =  cc_min_value; }  // fix minimal value
            if ( cc >=  cc_max_value ) { cc =  cc_max_value; }  // fix maximalvalue
            beat[channel].Velo_Morph_Offset = cc; }

            

    switch ( CC_Map_Mode ) { // 0: Deep Edit Mode    1: Over View Mode
        
        case 0: // Deep Edit Mode  -  1 Channel ALL Controlls
        
                // only Synth CCs  &&  NORMAL/MSQ MODE
                if ( last_enc < 32  &&  assign == 0 )  {
            
                        // save last moved CC
                        beat[Activ_CH].MSQ_LastMoved = last_enc;      // need later to delete MotionSequences on it
                    
                        // Change Store CC
                        cc = beat[Activ_CH].CC_Store[last_enc];
                        cc = cc + incrementer; 
                        if ( cc <=  cc_min_value ) { cc =  cc_min_value; }  // fix minimal value
                        if ( cc >=  cc_max_value ) { cc =  cc_max_value; }  // fix maximal value
                        beat[Activ_CH].CC_Store[last_enc] = cc;
            
                        // Clear Selected Motion Sequences 
                        if( MSQ_Clear_Sel == 1 )  { flag.msq_clear_sel = 1; }
                      
                        // Talk to SYNTH - Routing//                                        
                        // Send CC        Midi Port  Midi Channel        CC-Nr:  [Active Part Number] [last mooved CC Number]
                        u8 cc    = CC_Map0[Activ_CH][beat[Activ_CH].MSQ_LastMoved];
                        u8 value = beat[Activ_CH].CC_Store[beat[Activ_CH].MSQ_LastMoved];
                        if(cc  < 128) { MIOS32_MIDI_SendCC(MeloPort, MeloCh[Activ_CH],   cc,  value); }
                        if(cc == 254) { MIOS32_MIDI_SendCC(MeloPort, MeloCh[Activ_CH],   63,  63); //finetune fixed
                                        MIOS32_MIDI_SendCC(MeloPort, MeloCh[Activ_CH],   31,  value);} // tune
                        break; }

                // VELOCITY ASSIGN MODE
                if ( last_enc < 32  &&  assign == 1 )  {
            
                        VEL_LastMoved = last_enc; // needet to later delete Velocity Assignments
            
                        // write  CC into  Velocity Morph
                        cc = beat[channel].Velo_Morph[last_enc] + incrementer;
                        if ( cc <=  cc_min_value ) { cc =  cc_min_value; }  // fix minimal value
                        if ( cc >=  cc_max_value ) { cc =  cc_max_value; }  // fix maximal value
                        beat[channel].Velo_Morph[last_enc] = cc; break; }
            

                // Set  Active Part
                if ( last_enc == 33  &&  assign == 0 )  { 
                        cc = Activ_CH - incrementer;   // -incrementor because i reversed the pins hardware side...
                        if ( cc <  cc_min_value ) { cc =  cc_min_value; }  // fix minimal value
                        if ( cc >  cc_max_value ) { cc =  cc_max_value; }  // fix maximal value
                        Activ_CH = cc; break; }
                        

        default: // Over View Mode  -  best Controlls from 8 Channels  " C H A N N E L  S T R I P "
        
                // only Synth - Remote CCs
                if ( last_enc < 32  &&  assign == 0 ) {

                        // save last moved CC
                        beat[Strip_CH].MSQ_LastMoved = CC_Map1[Active_Strip_Set][last_enc];  // need later to delete MotionSequences on it
                        
                        // Store CC
                        cc = beat[Strip_CH].CC_Store  [ CC_Map1[Active_Strip_Set][last_enc] ];
                        cc = cc + incrementer;
                        if ( cc <=  cc_min_value ) { cc =  cc_min_value; }  // fix minimal value
                        if ( cc >=  cc_max_value ) { cc =  cc_max_value; }  // fix maximal value
                        beat[Strip_CH].CC_Store  [ CC_Map1[Active_Strip_Set][last_enc] ] = cc;

                        // Clear Selected Motion Sequences 
                        if( MSQ_Clear_Sel == 1 )  { flag.msq_clear_sel = 1; }
                        
                        u8 cc    = CC_Map0 [Strip_CH] [CC_Map1[Active_Strip_Set][last_enc] ];
                        u8 value = beat[Strip_CH].CC_Store  [CC_Map0 [Strip_CH] [CC_Map1[Active_Strip_Set][last_enc]] ];
                        if(cc  < 128) { MIOS32_MIDI_SendCC(MeloPort, MeloCh[Strip_CH],   cc,  value); }
                        if(cc == 254) { MIOS32_MIDI_SendCC(MeloPort, MeloCh[Strip_CH],   63,  63); //finetune fixed
                                        MIOS32_MIDI_SendCC(MeloPort, MeloCh[Strip_CH],   31,  value);} // tune
                        break; }                

                // set Active Strip Set              
                if ( last_enc == 33) { 
                    cc = Active_Strip_Set - incrementer; // -incrementor because i reversed the pins hardware side...
                    if ( cc <  cc_min_value ) { cc =  cc_min_value; }  // fix minimal value
                    if ( cc >  cc_max_value ) { cc =  cc_max_value; }  // fix maximal value
                    Active_Strip_Set = cc; break;}

                // VELOCITY ASSIGN MODE
                if ( last_enc < 32  &&  assign == 1 )  {
            
                        VEL_LastMoved = last_enc; // needet to later delete Velocity Assignments
            
                        // write  CC into  Velocity Morph
                        cc = beat[channel].Velo_Morph[ CC_Map1[Active_Strip_Set][last_enc] ] + incrementer;
                        if ( cc <=  cc_min_value ) { cc =  cc_min_value; }  // fix minimal value
                        if ( cc >=  cc_max_value ) { cc =  cc_max_value; }  // fix maximal value
                        beat[channel].Velo_Morph[ CC_Map1[Active_Strip_Set][last_enc] ] = cc; break; }

        break;
    } // END OFF  CC_Map_Mode  S W I T C H


}


static void TASK_SEQ(void *pvParameters){ // called each mS > seuecer request
  portTickType xLastExecutionTime;

  // Initialise the xLastExecutionTime variable on task entry
  xLastExecutionTime = xTaskGetTickCount();

  while( 1 ) {
    vTaskDelayUntil(&xLastExecutionTime, 1 / portTICK_RATE_MS);

    // execute sequencer handler
    SEQ_Handler();

  }
}

// SEQ --- Installed via MIOS32_MIDI_DirectRxCallback_Init
static s32 NOTIFY_MIDI_Rx(mios32_midi_port_t port, u8 midi_byte){//filter Midiclock Port and rerout it to SEQUENCER-BPM
if(port == MeloPort) { SEQ_BPM_NotifyMIDIRx(midi_byte); }
  return 0; // no error, no filtering
}



u8 MSQ(u8 port, u8 step) {  //  CC Automation - Motion Sequencer //505 can i avoid floating point here?
    
    // REC MSQ
    if(beat[port].MSQ_Rec == 1) {  beat[port].MSQ_Store[beat[channel].MSQ_LastMoved][step] = beat[channel].CC_Store[beat[channel].MSQ_LastMoved]; }

    // show Loop Position on a LED-RING
    IND = ((10000*loop_step_pos[channel]/(beat[0].PAGE_Steps*(beat[channel].loop_lenght_set+1))) * 15)/10000; // scale LoopLength to 0-15 LEDs   


 // Play MSQ @ SYNTH && ADD Velocity Offset
 if( port < 8  &&  beat[port].MSQ_Active == 1 ) {
        static float value = 0;
        static float morph = 0;
                  
        int x; for(x=0; x<32; x++) {

            value = beat[port].CC_Store[x]; // VALUE = Actually CC-VALUE of REMOTE
                      
            if( beat[port].MSQ_Store[x][step] != 128 ) { value =  beat[port].MSQ_Store[x][step]; } // overwrite  VALUE  with MotionSequence
            

            if( beat[port].MSQ_Store[x][step] != 128   ||   beat[port].Velo_Morph[x] != 63 ) { // no Motion Sequence   but  Velocity Offset
                
                    // calculate CC Value // avoid FLOATS  - 303 could be changed when FPU in MIOS would magicly work....   
                    morph = (((Velo_From_Note[port]/127.0) * (beat[port].Velo_Morph[x]-64.0))  / 64.0)   *   beat[port].Velo_Morph_Offset;
                    
                    value = value + morph;
                    
                    if(value <=   0) { value = 0; }    // only in a range of 0-127
                    if(value >= 127) { value = 127; }  // only in a range of 0-127
                    
                    int valueint = value;

                    //if(port == 0 ) { MIOS32_MIDI_SendDebugMessage("cc:%d, morph:%d, value:%d", x, morphint, valueint);}  // DEBUG
        
               //Talk to SYNTH   --- and ADD Velocity Morph!
                        u8 cc    = CC_Map0[port][x];
                        if(cc  < 128) { MIOS32_MIDI_SendCC(MeloPort, MeloCh[port],   cc,  valueint); }
                        if(cc == 254) { MIOS32_MIDI_SendCC(MeloPort, MeloCh[port],   63,  63); //finetune fixed
                                        MIOS32_MIDI_SendCC(MeloPort, MeloCh[port],   31,  valueint);} // tune
    }}
}
  return 0; // no error,
}


// S T O R A G E - SD - C A R D      ///        // read write clips/programchanges
void SD(u16 clip, u16 job){           // Job1=load, Job2=store, Job3=initalize card

    const static char file_type[4] = "bcr1";    // definie the Filetype which is later saved via "FILE_WriteBuffer" into the file header
    static char file_path[16] =  {};            // STORE-path - Number of Pathsymbols tm/512.tm    
    static s8 statusSD;			                // Status of the SDCard--connected? fat?
    static s8 statusDir;			            // Status "is" your direction "sdcard/t" there?
    static file_t midifile_fi;                  // used to read files from SD-Card
    
                             
    switch(job) {   
 
// R E A D /////////////////////////////////////////////////////////////////////////////////////////////////////// 
          
            case 1:
                    // L O A D Clip
                    sprintf         ( file_path, "bc/%d.bc", clip );
                    
                    MUTEX_SDCARD_TAKE;  
                        FILE_ReadOpen   ( &midifile_fi, file_path );
                        FILE_ReadBuffer ( (u8 *)&file_type, sizeof(file_type) ); 
                        FILE_ReadBuffer ( (u8 *)&beat, 8 * sizeof(store_t) );
                        FILE_ReadClose  ( &midifile_fi );
                    MUTEX_SDCARD_GIVE;
                    
                    flag.NeedSync = 1; //update ClockCounter when ready...
                    
                    flag.Update_Synths  = 1;
                break;
                	
                    
// S T O R E /////////////////////////////////////////////////////////////////////////////////////////////////////               
  
            case 2:
                    // S A V E Clip
                    sprintf(file_path, "bc/%d.bc", clip);
                     
                    MUTEX_SDCARD_TAKE;
                            //Write to File           
                            FILE_WriteOpen	( file_path,        sizeof(file_path) );   
                            FILE_WriteBuffer( (u8  *)&file_type,sizeof(file_type) ); //"SIGL" = 4 Positons                                                     
                            FILE_WriteBuffer( (u8  *)&beat, 8 * sizeof(store_t) );                                                             
                            FILE_WriteClose	(); 
                    MUTEX_SDCARD_GIVE;  //SD-Card is now free 4 access
                    
                break;
                    

//  I N I T - S D - CARD //////////////////////////////////////////////////////////////////////////////////7//////   
  // THIS WILL TAKE 10minutes//
            case 3: //check SD-Card
                    MUTEX_SDCARD_TAKE;					//SD-Card is now only for the following LINES reserved:
                      statusSD = FILE_CheckSDCard();
                      switch(statusSD) {
                          case 2:
                                // MIOS32_MIDI_SendDebugMessage("no card");
                                //LED_Gestic=3;
                            break;
                            
                          case 1: // YES CARD!  >>> next: check Card-content
                             statusDir = FILE_DirExists("bc");	 // ask file.c: exist a folder "t/" on the CARD? // 
                             if (statusDir == 1){                                         
                                    //MIOS32_MIDI_SendDebugMessage("SD OK");
                                    //LED_Gestic=0;
                                  break;} 
                             
                             if (statusDir == 0){
                                                                          
                                    //MIOS32_MIDI_SendDebugMessage("Init Card, take a minute");
                                    //LED_Gestic=1;
                                                      
            
                                        FILE_MakeDir("bc");                           

                                        // calculate file path
                                        sprintf(file_path, "bc/%d.bc", clip);
                                        
                                        //C L I P > Write to File           
                                        FILE_WriteOpen	( file_path,                sizeof(file_path) );                 
                                                FILE_WriteBuffer( (u8  *)&file_type,sizeof(file_type) ); //"SIGL" = 4 Positons     
                                                FILE_WriteBuffer( (u8  *)&beat, 8 * sizeof(store_t) ); 
                                        FILE_WriteClose	();

                                                            
                                            //copy-prototype file to 512x clip files on SD-Card
                                                u16 clipco;  // declare clip Counter 
                                                for(clipco = 0; clipco<256; clipco++){  //countes 2 511 and do following commandos 511 times in a loop:

                                                    char copyfilepath[16];
                                                    sprintf   (copyfilepath, "bc/%d.bc", clipco);  // make a new filename depending on the counter value 1.tm, 2.tm, 3.tm...511.tm
                                                    FILE_Copy ( (char *)"bc/0.bc", (char *)copyfilepath );} // copy the File 0.tm to all other 511 files...

                                    //MIOS32_MIDI_SendDebugMessage("Init Done");
                                    //LED_Gestic=2;
                                
                                    break;} 
                        }
                        MUTEX_SDCARD_GIVE;
                        }
}
